package org.ghisler.a7zip;

import android.annotation.TargetApi;
import android.app.Activity;
import android.content.ContentResolver;
import android.content.Context;
import android.net.Uri;
import android.os.Build;
import android.os.ParcelFileDescriptor;

import org.ghisler.tcpacker.IRemoteListCallback;
import org.ghisler.tcpacker.IRemoteUnpackCallback;

import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;

public class JniStreamCallback {
	public Context ctx;
	public IRemoteListCallback listCallback=null;
	public IRemoteUnpackCallback extractCallback=null;

	int MAXHANDLES=1000;  // max 999 part archive!
	ParcelFileDescriptor[] pfd=null;
	OutputStream[] ops=null;

	int getNextFreeHandle() {
		if (pfd==null) {
			pfd=new ParcelFileDescriptor[MAXHANDLES];
			ops=new OutputStream[MAXHANDLES];
			for (int i=0;i<MAXHANDLES;i++) {
				pfd[i]=null;
				ops[i]=null;
			}
			return 0;
		}
		for (int i=0;i<MAXHANDLES;i++) {
			if (pfd[i]==null)
				return i;
		}
		return -1;
	}

	public JniStreamCallback(Context ctx1) {
		ctx=ctx1;
	}

	public int openSourceDocument(String name) {
		int fh=getNextFreeHandle();
		if (fh<0)
			return 0;
		ContentResolver resolver = ctx.getContentResolver();
		try {
			if (listCallback!=null && listCallback.requestReadPermission(name) ||
				extractCallback!=null && extractCallback.requestReadPermission(name) ||
				listCallback==null && extractCallback==null)  // internal usage
				pfd[fh]=resolver.openFileDescriptor(Uri.parse(name), "r");
			else
				return 0;
			ops[fh] = null;
		} catch (Throwable e) {
			return 0;
		}
		return fh+1;
	}

	public int openTargetDocument(String name) {
		int fh=getNextFreeHandle();
		if (fh<0)
			return 0;
		ContentResolver resolver = ctx.getContentResolver();
		try {
			pfd[fh]=resolver.openFileDescriptor(Uri.parse(name), "w");
			ops[fh] = resolver.openOutputStream(Uri.parse(name));
			ops[fh] = null;
		} catch (Throwable e) {
			return 0;
		}
		return fh+1;
	}

	public int writeTargetDocument(int fileHandle,byte[] buffer,int len) {
		fileHandle--;   // it's one more!
		if (fileHandle<0 || fileHandle>=MAXHANDLES)
			return -1;
		if (pfd==null || pfd[fileHandle]==null)
			return -1;
		if (len==0)
			return 0;
		try {
			if (ops[fileHandle]==null)
				ops[fileHandle] = new BufferedOutputStream(new FileOutputStream(pfd[fileHandle].getFileDescriptor()));
			ops[fileHandle].write(buffer,0,len);
		} catch (Throwable e) {
			return -1;
		}
		return len;
	}

	public void closeDocument(int fileHandle) {
		fileHandle--;   // it's one more!
		if (fileHandle<0 || fileHandle>=MAXHANDLES)
			return;
		try {
			if (ops[fileHandle]!=null) {
				ops[fileHandle].flush();
				ops[fileHandle].close();
			}
		} catch (Throwable e) {
		}
		try {
			if (pfd[fileHandle]!=null)
				pfd[fileHandle].close();
		} catch (Throwable e) {
		}
		ops[fileHandle]=null;
		pfd[fileHandle]=null;
	}

	@TargetApi(Build.VERSION_CODES.HONEYCOMB_MR1)
	public int getFd(int fileHandle) {
		fileHandle--;   // it's one more!
		if (fileHandle<0 || fileHandle>=MAXHANDLES)
			return 0;
		try {
			if (pfd!=null && pfd[fileHandle]!=null)
				return pfd[fileHandle].getFd();
		} catch (Throwable e) {
		}
		return 0;
	}

}
