package org.ghisler.a7zip;

import android.app.Activity;
import android.content.ContentResolver;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.view.View;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Toast;

import org.w3c.dom.Text;

import java.util.ArrayList;
import java.util.List;

public class MainActivity extends Activity {

	private static final int saltsize = 32;
	private static final int REQUEST_OPEN_FOLDER = 1;
	private static final int REQUEST_RW_RIGHTS = 2;
	static int EXTRACT_EXTRACT = 0;
	static int EXTRACT_SKIP = 1;
	static int EXTRACT_ABORT = 2;
	static MainActivity app = null;
	static JniStreamCallback jsc = null;
	private Un7zipjniLib Un7zip = null;
	private Uri uri = null;
	private boolean busy=false;
	private boolean aborted=false;

	public static MainActivity getApp() {
		return app;
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		app = this;
		jsc = new JniStreamCallback(this);
		setContentView(R.layout.activity_main);

		Button pick = (Button) findViewById(R.id.pick);
		pick.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				if (busy) {
					aborted=true;
					return;
				}
				Intent intent = new Intent(Intent.ACTION_OPEN_DOCUMENT);
				intent.setType("*/*");
				intent.addCategory(Intent.CATEGORY_OPENABLE);
				intent.setFlags(Intent.FLAG_GRANT_PERSISTABLE_URI_PERMISSION);
				try {
					startActivityForResult(intent, REQUEST_OPEN_FOLDER);
				} catch (Throwable e) {
					Toast.makeText(MainActivity.this,"File picker not supported on this system!",Toast.LENGTH_LONG).show();
				}
			}
		});
		Button test = (Button) findViewById(R.id.test);
		test.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				if (busy) {
					aborted=true;
					return;
				}
				TextView tv = (TextView) findViewById(R.id.output);
				tv.setText("Testing...");
				busy=true;
				aborted=false;
				Thread thread=new Thread(new Runnable() {
					@Override
					public void run() {
						unpackJni(true);
						busy=false;
					}
				});
				thread.start();
			}
		});
		Button extract = (Button) findViewById(R.id.unpack);
		extract.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				if (busy) {
					aborted=true;
					return;
				}
				TextView tv = (TextView) findViewById(R.id.output);
				tv.setText("Unpacking...");
				busy=true;
				aborted=false;
				Thread thread=new Thread(new Runnable() {
					@Override
					public void run() {
						unpackJni(false);
						busy=false;
					}
				});
				thread.start();
			}
		});
		Button list = (Button) findViewById(R.id.list);
		list.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View view) {
				if (busy) {
					aborted=true;
					return;
				}
				TextView tv = (TextView) findViewById(R.id.output);
				tv.setText("Getting list of files...");
				busy=true;
				aborted=false;
				Thread thread=new Thread(new Runnable() {
					@Override
					public void run() {
						listJni();
						busy=false;
					}
				});
				thread.start();
			}
		});

		try {  // catch crash if Android M functions missing
			if (Build.VERSION.SDK_INT >= 23 && !AndroidMFunctions.hasStorageReadWriteRights(this)) {
				AndroidMFunctions.requestStorageReadWriteRights(this,REQUEST_RW_RIGHTS);
			}
		} catch (Throwable e) {}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == REQUEST_OPEN_FOLDER && data!=null) {
			ContentResolver resolver = getContentResolver();
			uri = data.getData();
		}
	}

	private void listJni() {
		final String[] result=new String[2];
		result[0]="";
		result[1]="";
		String sdcard = Environment.getExternalStorageDirectory().getAbsolutePath();
		if (!sdcard.endsWith("/"))
			sdcard+="/";

		try {
			if (Un7zip==null)
				Un7zip=new Un7zipjniLib();
		} catch (UnsatisfiedLinkError e) {
			return;
		} catch (Throwable e2) {
			return;
		}
		String name;
		if (uri!=null)
			name=uri.toString();
		else
			name = sdcard + "temp/7z/7z_test.7z";

		aborted=false;
		int err7=Un7zip.list(name, new I7zipListCallback() {
			int total=0;
			int lastPercent=-1;

			@Override
			public void onGetFileCount(int fileCount) {
				result[0]+="Number of files:"+fileCount+(fileCount>10000?" (listing first 10000)":"")+"\n";
				total=fileCount;
			}

			@Override
			public int onListItem(int index, String fullPath, boolean isDir, boolean isEncrypted, long size, long ntfstime) {
				result[1]+=fullPath+" "+(isDir?"<folder>":size+"b")+"\n";
				if (result[1].length()>65535) {  // much faster for many files
					result[0]+=result[1];
					result[1]="";
				}
				if (total>0) {
					final int percent=(int)(index*100/(total>10000?10000:total));
					if (percent!=lastPercent) {
						lastPercent = percent;
						runOnUiThread(new Runnable() {
							@Override
							public void run() {
								ProgressBar pb = (ProgressBar) findViewById(R.id.progressBar);
								if (pb != null)
									pb.setProgress(percent);
							}
						});
					}
				}
				return aborted || (index>=10000)?1:0;
			}

			@Override
			public String onGetPassword() {
				return null;
			}
		},jsc);
		if (err7==0)
			result[0] = "7zip list OK!\n"+result[0]+result[1];
		else if ((err7&8)!=0)
			result[0] = "7zip list Aborted!\n"+result[0]+result[1];
		else
			result[0] = "7zip list error "+err7+"\n"+result[0]+result[1];
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				TextView tv = (TextView) findViewById(R.id.output);
				tv.setText(result[0]);
			}
		});
	}

	private void unpackJni(boolean test) {
		final String[] result=new String[1];
		result[0]="";
		String sdcard = Environment.getExternalStorageDirectory().getAbsolutePath();
		if (!sdcard.endsWith("/"))
			sdcard+="/";

		try {
			if (Un7zip==null)
				Un7zip=new Un7zipjniLib();
		} catch (UnsatisfiedLinkError e) {
			Un7zip=null;
			//Utilities.longToast(TcApplication.getApp(),"Link error loading UNRAR library!\n"+e.getMessage());
		} catch (Throwable e2) {
			Un7zip=null;
			//Utilities.longToast(TcApplication.getApp(),"General error loading UNRAR library!\n"+e2.getMessage());
		}
		String name;
		String destination;
		if (uri!=null)
			name=uri.toString();
		else
			name = sdcard + "temp/7z/7z_test.7z";
		if (test)
			destination = "/dev/null/";
		else
			destination = sdcard + "test/out/";

		final String archive=name;
		int err7=Un7zip.extract(name,0,destination,new I7zipUnpackCallback() {
			@Override
			public void onGetFileCount(int fileCount) {
				result[0]+="Number of files:"+fileCount+"\n";
			}

			@Override
			public int onUnpackProgress(long part, long total) {
				if (total>0) {
					final int percent=(int)(part*100/total);
					runOnUiThread(new Runnable() {
						@Override
						public void run() {
							ProgressBar pb=(ProgressBar)findViewById(R.id.progressBar);
							if (pb!=null)
								pb.setProgress(percent);
						}
					});
				}
				return aborted?1:0;  // continue=0, abort=1
			}

			@Override
			public String onWantExtract(int index, String fullPath, String diskFilePath, boolean isDir, boolean isEncrypted, long size, long ntfstime) {
				result[0]+=fullPath+"\n";
				if (aborted)
					return null;
				else if (archive.toLowerCase().endsWith(".iso") && fullPath.startsWith(".disk"))
					return "";   // skip metadata in iso, unpacking fails!
				else
					return diskFilePath;
			}

			@Override
			public String onGetPassword() {
				return null;
			}

			@Override
			public int onCreateDir(String name) {
				return 0;
			}
		},jsc);
		if (err7==0)
			result[0] = "7zip OK!\n"+result[0];
		else
			result[0] = "7zip error "+err7+"\n"+result[0];
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				TextView tv = (TextView) findViewById(R.id.output);
				tv.setText(result[0]);
			}
		});
	}
}
