package org.ghisler.tcpacker;

import android.os.IBinder;
import android.os.RemoteException;
import android.os.SystemClock;
import android.widget.ProgressBar;

import org.ghisler.a7zip.I7zipListCallback;
import org.ghisler.a7zip.I7zipUnpackCallback;
import org.ghisler.a7zip.JniStreamCallback;
import org.ghisler.a7zip.R;
import org.ghisler.a7zip.Un7zipjniLib;
import org.ghisler.tcpacker.IPackerPluginFunctions;

public class PackerPluginFunctions extends IPackerPluginFunctions.Stub {
	PluginService service;
	private Un7zipjniLib Un7zip = null;
	static JniStreamCallback jsc = null;

	private final int UN7ZIPERROR_UNSUPPORTED=1;
	private final int UN7ZIPERROR_CRCERROR=2;
	private final int UN7ZIPERROR_DATAERROR=4;
	private final int UN7ZIPERROR_ABORTED=8;
	private final int UN7ZIPERROR_CREATE=16;
	private final int UN7ZIPERROR_DELETE=32;
	private final int UN7ZIPERROR_MEMORY=64;
	private final int UN7ZIPERROR_INTERNAL=128;

	public PackerPluginFunctions(PluginService service2) {
		service = service2;
	}

	@Override
	public int list(String name, final IRemoteListCallback callback) throws RemoteException {
		if (jsc==null)
			jsc = new JniStreamCallback(service);
		try {
			if (Un7zip==null)
				Un7zip=new Un7zipjniLib();
		} catch (UnsatisfiedLinkError e) {
			Un7zip=null;
			//Utilities.longToast(TcApplication.getApp(),"Link error loading UNRAR library!\n"+e.getMessage());
		} catch (Throwable e2) {
			Un7zip=null;
			//Utilities.longToast(TcApplication.getApp(),"General error loading UNRAR library!\n"+e2.getMessage());
		}
		if (Un7zip==null)
			return UN7ZIPERROR_INTERNAL;
		if (name.length()>=2 && name.charAt(0)=='/' && name.charAt(1)!='/')
			if (!PluginService.requestWriteRightsAndWait(service))
				return UN7ZIPERROR_DATAERROR;

		jsc.listCallback=callback;
		jsc.extractCallback=null;
		int err7=Un7zip.list(name, new I7zipListCallback() {
			int total=0;
			int lastPercent=-1;

			@Override
			public void onGetFileCount(int fileCount) {
				try {
					callback.onGetFileCount(fileCount);
				} catch (Throwable e) {
				}
			}

			@Override
			public int onListItem(int index, String fullPath, boolean isDir, boolean isEncrypted, long size, long ntfstime) {
				long filetime=ntfstime/10000-11644473600000L;
				try {
					return callback.onListItem(index,fullPath,isDir,isEncrypted,size,filetime);
				} catch (Throwable e) {
					return 1;  //abort due to remote exception
				}
			}

			@Override
			public String onGetPassword() {
				try {
					return callback.onGetPassword();
				} catch (Throwable e) {
					return null;
				}
			}
		},jsc);
		jsc.listCallback=null;
		return err7;
	}

	long lasttime;

	@Override
	public int extract(String name, int subStart, String destination, final IRemoteUnpackCallback callback) throws RemoteException {
		if (jsc==null)
			jsc = new JniStreamCallback(service);
		try {
			if (Un7zip==null)
				Un7zip=new Un7zipjniLib();
		} catch (UnsatisfiedLinkError e) {
			Un7zip=null;
			//Utilities.longToast(TcApplication.getApp(),"Link error loading UNRAR library!\n"+e.getMessage());
		} catch (Throwable e2) {
			Un7zip=null;
			//Utilities.longToast(TcApplication.getApp(),"General error loading UNRAR library!\n"+e2.getMessage());
		}
		if (Un7zip==null)
			return UN7ZIPERROR_INTERNAL;
		if (name.length()>=2 && name.charAt(0)=='/' && name.charAt(1)!='/') {
			if (!PluginService.requestWriteRightsAndWait(service))
				return UN7ZIPERROR_DATAERROR;
		} else if (destination!=null && destination.length()>=2 && destination.charAt(0)=='/' && destination.charAt(1)!='/')
			if (!PluginService.requestWriteRightsAndWait(service))
				return UN7ZIPERROR_CREATE;

		jsc.listCallback=null;
		jsc.extractCallback=callback;
		lasttime=SystemClock.currentThreadTimeMillis();
		int err7=Un7zip.extract(name,subStart,destination,new I7zipUnpackCallback() {
			@Override
			public void onGetFileCount(int fileCount) {
				try {
					callback.onGetFileCount(fileCount);
				} catch (Throwable e) {
				}
			}

			@Override
			public int onUnpackProgress(long part, long total) {
				try {
					long thistime=SystemClock.currentThreadTimeMillis();
					if (thistime-lasttime>500 || part==total) {
						lasttime=thistime;
						return callback.onUnpackProgress(part, total);
					} else
						return 0;
				} catch (Throwable e) {
					return 1;  // continue=0, abort=1
				}
			}

			@Override
			public String onWantExtract(int index, String fullPath, String diskFilePath, boolean isDir, boolean isEncrypted, long size, long ntfstime) {
				try {
					long filetime=ntfstime/10000-11644473600000L;
					return callback.onWantExtract(index,fullPath,diskFilePath,isDir,isEncrypted,size,filetime);
				} catch (Throwable e) {
					return null;
				}
			}

			@Override
			public String onGetPassword() {
				try {
					return callback.onGetPassword();
				} catch (Throwable e) {
					return null;
				}
			}

			@Override
			public int onCreateDir(String name) {
				try {
					return callback.onCreateDir(name);
				} catch (Throwable e) {
					return 0;
				}
			}
		},jsc);
		jsc.extractCallback=null;
		return err7;
	}

}
